import warnings
import numpy as np
from scipy import signal
import matplotlib
import matplotlib.pylab as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable
import matplotlib.transforms as transforms
from matplotlib.patches import Rectangle

class SpikeCalcsGeneric(object):
	"""
	Deals with the processing and analysis of spike data.

	Parameters
	----------
	spike_times : array_like
		The times of 'spikes' in the trial
		Should be the same length as the cluster identity vector _spk_clusters
	waveforms : np.array, optional
		An nSpikes x nSamples array (nSamples usually 50)

	Notes
	-----
	Units for time are provided as per the sample rate but converted internally to milliseconds
	"""
	def __init__(self, spike_times, waveforms=None, **kwargs):
		self.spike_times = spike_times
		self.waveforms = waveforms
		self._event_ts = None # the times that events occured i.e. the laser came on
		self._spk_clusters = None # vector of cluster ids, same length as spike_times
		self._event_window = np.array((-0.050, 0.100)) # window, in seconds, either side of the stimulus, to examine
		self._stim_width = None # the width, in ms, of the stimulus
		self._secs_per_bin = 0.001 # used to increase / decrease size of bins in psth
		self._sample_rate = 30000
		self._duration = None
		self._pre_spike_samples = 18
		self._pos_spike_samples = 32

	@property
	def sample_rate(self):
		return self._sample_rate

	@sample_rate.setter
	def sample_rate(self, value):
		self._sample_rate = value
	
	@property
	def pre_spike_samples(self):
		return self._pre_spike_samples
	
	@pre_spike_samples.setter
	def pre_spike_samples(self, value):
		self._pre_spike_samples = int(self._pre_spike_samples)

	@property
	def post_spike_samples(self):
		return self._post_spike_samples
	
	@post_spike_samples.setter
	def post_spike_samples(self, value):
		self._post_spike_samples = int(self._post_spike_samples)

	def n_spikes(self, cluster=None):
		if cluster is None:
			return len(self.spike_times)
		else:
			if self.spk_clusters is None:
				warnings.warn("No clusters available, please load some into me.")
				return
			else:
				return np.count_nonzero(self._spk_clusters==cluster)

	@property
	def event_ts(self):
		return self._event_ts

	@event_ts.setter
	def event_ts(self, value):
		self._event_ts = value

	@property
	def duration(self):
		return self._duration

	@duration.setter
	def duration(self, value):
		self._duration = value

	@property
	def spk_clusters(self):
		return self._spk_clusters

	@spk_clusters.setter
	def spk_clusters(self, value):
		self._spk_clusters = value

	@property
	def event_window(self):
		return self._event_window

	@event_window.setter
	def event_window(self, value):
		self._event_window = value

	@property
	def stim_width(self):
		return self._stim_width

	@stim_width.setter
	def stim_width(self, value):
		self._stim_width = value

	@property
	def _secs_per_bin(self):
		return self.__secs_per_bin

	@_secs_per_bin.setter
	def _secs_per_bin(self, value):
		self.__secs_per_bin = value

	def trial_mean_fr(self, cluster: int)->float:
		# Returns the trial mean firing rate for the cluster
		if self.duration is None:
			warnings.warn("No duration provided, give me one!")
			return
		return self.n_spikes(cluster) / self.duration

	def mean_isi_range(self, cluster: int, n: int)->float:
		"""
		Calculates the mean of the autocorrelation from 0 to n milliseconds
		Used to help classify a neruons type (principal, interneuron etc)

		Parameters
		----------
		cluster : int
			The cluster to analyse
		n : int
			The range in milliseconds to calculate the mean over

		Returns
		-------
		mean_isi_range : float
			The mean of the autocorrelogram between 0 and n milliseconds
		"""
		if cluster not in self.spk_clusters:
			warnings.warn("Cluster not available")
			return
		bins = 201
		trange = np.array((-500, 500))
		t = self.spike_times[self.spk_clusters==cluster]
		y = self.xcorr(t, Trange=trange)
		y = y.astype(np.int64) # See xcorr docs
		counts, bins = np.histogram(y[y!=0], bins=bins, range=trange)
		mask = np.logical_and(bins>0, bins<n)
		return np.mean(counts[mask[1:]])

	def xcorr(self, x1: np.ndarray, x2=None, Trange=None, **kwargs)->np.ndarray:
		"""
		Calculates the histogram of the ISIs in x1 or x1 vs x2

		Parameters
		----------
		x1, x2 : array_like
			The times of the spikes emitted by the cluster(s)
			NB must be signed int to accomodate negative times
		Trange : array_like
			Range of times to bin up. Defaults to [-500, +500] in ms

		Returns
		-------
		y : np.ndarray
			The time differences between spike times in x1 over the range
			of times defined Trange
		"""
		if x2 is None:
			x2 = x1.copy()
		if Trange is None:
			Trange = np.array([-500, 500])
		if type(Trange) == tuple:
			Trange = np.array(Trange)
		y = []
		irange = x1[:, np.newaxis] + Trange[np.newaxis, :]
		dts = np.searchsorted(x2, irange)
		for i, t in enumerate(dts):
			y.extend(x2[t[0]:t[1]] - x1[i])
		y = np.array(y, dtype=float)
		return y

	def getClusterWaveforms(self, cluster_id: int, channel_id: int):
		"""
		Get the waveforms for a particular cluster on a given channel

		Parameters
		----------
		cluster_id : int

		Returns
		-------
		np.array : the waveforms
		"""
		if self.waveforms is not None:
			pass
		else:
			return None

	def getMeanWaveform(self, cluster_id: int, channel_id: int):
		"""
		Returns the mean waveform and sem for a given spike train
		on a particular channel
		
		Parameters
		----------
		cluster_id: int
			The cluster to get the mean waveform for
			
		Returns
		-------
		mn_wvs: ndarray (floats) - usually 4x50 for tetrode recordings
			the mean waveforms
		std_wvs: ndarray (floats) - usually 4x50 for tetrode recordings
			the standard deviations of the waveforms
		"""
		if self.waveforms and self.spk_clusters is not None:
			if cluster_id not in self.spk_clusters:
				warnings.warn('Cluster not available. Try again!')
			x = self.getClusterWaveforms(cluster_id, channel_id)
			if x is not None:
				return np.mean(x, axis=0), np.std(x, axis=0)
			else:
				return None
		else:
			return None

	def calculatePSTH(self, cluster_id, **kwargs):
		"""
		Calculate the PSTH of event_ts against the spiking of a cell

		Parameters
		----------
		cluster_id : int
			The cluster for which to calculate the psth

		Returns
		-------
		x, y : list
			The list of time differences between the spikes of the cluster
			and the events (x) and the trials (y)
		"""
		if self._event_ts is None:
			raise Exception("Need some event timestamps! Aborting")
		if self._spk_clusters is None:
			raise Exception("Need cluster identities! Aborting")
		event_ts = self.event_ts
		event_ts.sort()
		if type(event_ts) == list:
			event_ts = np.array(event_ts)

		spike_times = self.spike_times[self.spk_clusters == cluster_id]
		irange = event_ts[:, np.newaxis] + self.event_window[np.newaxis, :]
		dts = np.searchsorted(spike_times, irange)
		x = []
		y = []
		for i, t in enumerate(dts):
			tmp = spike_times[t[0]:t[1]] - event_ts[i]
			x.extend(tmp)
			y.extend(np.repeat(i,len(tmp)))
		return x, y

	def plotPSTH(self, cluster, fig=None):
		"""
		Plots the PSTH for a cluster

		Parameters
		----------
		cluster : int
			The cluster to examine

		Returns
		-------
		cluster, i : int
			The cluster and a junk variable (not sure why for now)
		"""
		x, y = self.calculatePSTH(cluster)
		show = False # used below to show the figure or leave this to the caller
		if fig is None:
			fig = plt.figure(figsize=(4.0,7.0))
			show = True
		scatter_ax = fig.add_subplot(111)
		scatter_ax.scatter(x, y, marker='.', s=2, rasterized=False)
		divider = make_axes_locatable(scatter_ax)
		scatter_ax.set_xticks((self.event_window[0], 0, self.event_window[1]))
		scatter_ax.set_xticklabels((str(self.event_window[0]), '0', str(self.event_window[1])))
		hist_ax = divider.append_axes("top", 0.95, pad=0.2, sharex=scatter_ax,
									  transform=scatter_ax.transAxes)
		scattTrans = transforms.blended_transform_factory(scatter_ax.transData,
														  scatter_ax.transAxes)
		if self.stim_width is not None:
			scatter_ax.add_patch(Rectangle((0, 0), width=self.stim_width, height=1,
						transform=scattTrans,
						color=[0, 0, 1], alpha=0.5))
			histTrans = transforms.blended_transform_factory(hist_ax.transData,
															 hist_ax.transAxes)
			hist_ax.add_patch(Rectangle((0, 0), width=self.stim_width, height=1,
							  transform=histTrans,
							  color=[0, 0, 1], alpha=0.5))
		scatter_ax.set_ylabel('Laser stimulation events', labelpad=-18.5)
		scatter_ax.set_xlabel('Time to stimulus onset(secs)')
		nStms = int(len(self.event_ts))
		scatter_ax.set_ylim(0, nStms)
		# Label only the min and max of the y-axis
		ylabels = scatter_ax.get_yticklabels()
		for i in range(1, len(ylabels)-1):
			ylabels[i].set_visible(False)
		yticks = scatter_ax.get_yticklines()
		for i in range(1, len(yticks)-1):
			yticks[i].set_visible(False)
		histColor = [192/255.0,192/255.0,192/255.0]
		hist_ax.hist(x, bins=np.arange(self.event_window[0], self.event_window[1] + self._secs_per_bin, self._secs_per_bin),
							 color=histColor, alpha=0.6, range=self.event_window, rasterized=True, histtype='stepfilled')
		hist_ax.set_ylabel("Spike count", labelpad=-2.5)
		plt.setp(hist_ax.get_xticklabels(), visible=False)
		# Label only the min and max of the y-axis
		ylabels = hist_ax.get_yticklabels()
		for i in range(1, len(ylabels)-1):
			ylabels[i].set_visible(False)
		yticks = hist_ax.get_yticklines()
		for i in range(1, len(yticks)-1):
			yticks[i].set_visible(False)
		hist_ax.set_xlim(self.event_window)
		scatter_ax.set_xlim(self.event_window)
		if show:
			plt.show()
		yield cluster, 1

	def plotAllXCorrs(self, clusters, fig=None):
		"""
		Plots all xcorrs in a single figure window

		Parameters
		----------
		clusters : list
			The clusters to plot
		fig : matplotlib.figure instance, optional, default None
			If provided the figure will contain all the axes
		"""
		if fig is None:
			fig = plt.figure(figsize=(10,20))

		nrows = np.ceil(np.sqrt(len(clusters))).astype(int)
		fig.subplots_adjust(wspace=0.25,hspace=0.25)
		for i, cluster in enumerate(clusters):
			cluster_idx = np.nonzero(self.spk_clusters == cluster)[0]
			cluster_ts = np.ravel(self.spike_times[cluster_idx])
			ax = fig.add_subplot(nrows,nrows,i+1)
			y = self.xcorr(cluster_ts.T / float(self.sample_rate / 1000)) # ms
			ax.hist(y[y != 0], bins=201, range=[-500, 500], color='k', histtype='stepfilled')
			ax.set_xlim(-500,500)
			ax.set_xticks((-500, 0, 500))
			# ax.set_xticklabels((str(-500), '0', str(500)),fontweight='normal', size=8)
			ax.set_xticklabels('')
			ax.tick_params(axis='both', which='both', left=False, right=False,
							bottom=False, top=False)
			ax.set_yticklabels('')
			ax.spines['right'].set_visible(False)
			ax.spines['top'].set_visible(False)
			ax.spines['left'].set_visible(False)
			ax.xaxis.set_ticks_position('bottom')
			ax.set_title(cluster, fontweight='bold', size=10, pad=1)
		plt.show()
		
	def thetaModIdx(self, x1, **kwargs):
		"""
		Calculates a theta modulation index of a spike train based on the cells
		autocorrelogram
		
		Parameters
		----------
		x1: np.array
			The spike time-series
		Returns
		-------
		thetaMod: float
			The difference of the values at the first peak and trough of the
			autocorrelogram
		"""
		y = self.xcorr(x1)
		corr, _ = np.histogram(y[y != 0], bins=201, range=np.array([-500,500]))
		# Take the fft of the spike train autocorr (from -500 to +500ms)
		from scipy.signal import periodogram
		if 'fs' in kwargs:
			fs = kwargs['fs']
		else:
			fs = 200
		freqs, power = periodogram(corr, fs=fs, return_onesided=True)
		# Smooth the power over +/- 1Hz
		b = signal.boxcar(3)
		h = signal.filtfilt(b, 3, power)
		
		# Square the amplitude first
		sqd_amp = h ** 2
		# Then find the mean power in the +/-1Hz band either side of that
		theta_band_max_idx = np.nonzero(sqd_amp==np.max(sqd_amp[np.logical_and(freqs>6, freqs<11)]))[0][0]
		mean_theta_band_power = np.mean(sqd_amp[theta_band_max_idx-1:theta_band_max_idx+1])
		# Find the mean amplitude in the 2-50Hz range
		other_band_idx = np.logical_and(freqs>2, freqs<50)
		mean_other_band_power = np.mean(sqd_amp[other_band_idx])
		# Find the ratio of these two - this is the theta modulation index
		return (mean_theta_band_power - mean_other_band_power) / (mean_theta_band_power + mean_other_band_power)

	def thetaModIdxV2(self, x1):
		"""
		This is a simpler alternative to the thetaModIdx method in that it
		calculates the difference between the normalized temporal autocorrelogram
		at the trough between 50-70ms and the peak between 100-140ms over
		their sum (data is binned into 5ms bins)
		
		Measure used in Cacucci et al., 2004 and Kropff et al 2015
		"""
		y = self.xcorr(x1)
		corr, bins = np.histogram(y[y != 0], bins=201, range=np.array([-500,500]))
		# 'close' the right-hand bin
		bins = bins[0:-1]
		# normalise corr so max is 1.0
		corr = corr/float(np.max(corr))
		thetaAntiPhase = np.min(corr[np.logical_and(bins>50,bins<70)])
		thetaPhase = np.max(corr[np.logical_and(bins>100, bins<140)])
		return (thetaPhase-thetaAntiPhase) / (thetaPhase+thetaAntiPhase)

	def clusterQuality(self, cluster, fet=1):
		"""
		returns the L-ratio and Isolation Distance measures
		calculated on the principal components of the energy in a spike matrix
		"""
		if self.waveforms is None:
			return None
		nSpikes, nElectrodes, _ = self.waveforms.shape
		wvs = self.waveforms.copy()
		E = np.sqrt(np.nansum(self.waveforms ** 2, axis=2))
		zeroIdx = np.sum(E, 0) == [0, 0, 0, 0]
		E = E[:, ~zeroIdx]
		wvs = wvs[:, ~zeroIdx, :]
		normdWaves = (wvs.T / E.T).T
		PCA_m = self.getParam(normdWaves, 'PCA', fet=fet)
		# get mahalanobis distance
		idx = self.cut == cluster
		nClustSpikes = np.count_nonzero(idx)
		try:
			d = self._mahal(PCA_m,PCA_m[idx,:])
			# get the indices of the spikes not in the cluster
			M_noise = d[~idx]
			df = np.prod((fet, nElectrodes))
			from scipy import stats
			L = np.sum(1 - stats.chi2.cdf(M_noise, df))
			L_ratio = L / nClustSpikes
			# calculate isolation distance
			if nClustSpikes < nSpikes / 2:
				M_noise.sort()
				isolation_dist = M_noise[nClustSpikes]
			else:
				isolation_dist = np.nan
		except:
			isolation_dist = L_ratio = np.nan
		return L_ratio, isolation_dist

	def _mahal(self, u, v):
		"""
		gets the mahalanobis distance between two vectors u and v
		a blatant copy of the Mathworks fcn as it doesn't require the covariance
		matrix to be calculated which is a pain if there are NaNs in the matrix
		"""
		u_sz = u.shape
		v_sz = v.shape
		if u_sz[1] != v_sz[1]:
			warnings.warn('Input size mismatch: matrices must have same number of columns')
		if v_sz[0] < v_sz[1]:
			warnings.warn('Too few rows: v must have more rows than columns')
		if np.any(np.imag(u)) or np.any(np.imag(v)):
			warnings.warn('No complex inputs are allowed')
		m = np.nanmean(v,axis=0)
		M = np.tile(m, reps=(u_sz[0],1))
		C = v - np.tile(m, reps=(v_sz[0],1))
		_, R = np.linalg.qr(C)
		ri = np.linalg.solve(R.T, (u-M).T)
		d = np.sum(ri * ri,0).T * (v_sz[0]-1)
		return d

class SpikeCalcsTetrode(SpikeCalcsGeneric):
	"""
	Encapsulates methods specific to the geometry inherent in tetrode-based
	recordings
	"""
	def __init__(self):
		pass

class SpikeCalcsAxona(SpikeCalcsGeneric):
	"""
	Replaces SpikeCalcs from ephysiopy.dacq2py.spikecalcs
	"""
	def getParam(self, waveforms=None, param='Amp', t=200, fet=1):
		"""
		Returns the requested parameter from a spike train as a numpy array
		
		Parameters
		-------------------
		
		waveforms - numpy array		
			Shape of array can be an nSpikes x nSamples
			OR
			a nSpikes x nElectrodes x nSamples
		
		param - str
			Valid values are:
				'Amp' - peak-to-trough amplitude (default)
				'P' - height of peak
				'T' - depth of trough
				'Vt' height at time t
				'tP' - time of peak (in seconds)
				'tT' - time of trough (in seconds)
				'PCA' - first n fet principal components (defaults to 1)
				
		t - int
			The time used for Vt
			
		fet - int
			The number of principal components (used with param 'PCA')
		"""
		from scipy import interpolate
		from sklearn.decomposition import PCA
		
		if waveforms is None:
			waveforms = self.waveforms
			
		if param == 'Amp':
			return np.ptp(waveforms, axis=-1)
		elif param == 'P':
			return np.max(waveforms, axis=-1)
		elif param == 'T':
			return np.min(waveforms, axis=-1)
		elif param == 'Vt':
			times = np.arange(0,1000,20)
			f = interpolate.interp1d(times, range(50), 'nearest')
			if waveforms.ndim == 2:
				return waveforms[:, int(f(t))]
			elif waveforms.ndim == 3:
				return waveforms[:, :, int(f(t))]
		elif param == 'tP':
			idx = np.argmax(waveforms, axis=-1)
			m = interpolate.interp1d([0, waveforms.shape[-1]-1], [0, 1/1000.])
			return m(idx)
		elif param == 'tT':
			idx = np.argmin(waveforms, axis=-1)
			m = interpolate.interp1d([0, waveforms.shape[-1]-1], [0, 1/1000.])
			return m(idx)
		elif param == 'PCA':
			pca = PCA(n_components=fet)
			if waveforms.ndim == 2:
				return pca.fit(waveforms).transform(waveforms).squeeze()
			elif waveforms.ndim == 3:
				out = np.zeros((waveforms.shape[0], waveforms.shape[1] * fet))
				st = np.arange(0, waveforms.shape[1] * fet, fet)
				en = np.arange(fet, fet + (waveforms.shape[1] * fet), fet)
				rng = np.vstack((st, en))
				for i in range(waveforms.shape[1]):
					if ~np.any(np.isnan(waveforms[:,i,:])):
						A = np.squeeze(pca.fit(waveforms[:,i,:].squeeze()).transform(waveforms[:,i,:].squeeze()))
						if A.ndim < 2:
							out[:,rng[0,i]:rng[1,i]] = np.atleast_2d(A).T
						else:
							out[:,rng[0,i]:rng[1,i]] = A
				return out

	def half_amp_dur(self, cluster):
		"""
		Half amplitude duration of a spike
		
		Parameters
		----------
		A: ndarray
			An nSpikes x nElectrodes x nSamples array
			
		Returns
		-------
		had: float
			The half-amplitude duration for the channel (electrode) that has 
			the strongest (highest amplitude) signal. Units are ms
		"""
		from scipy import interpolate, optimize
		
		waveforms = self.waveforms[self.cut==cluster, :, :]
		best_chan = np.argmax(np.max(np.mean(waveforms, 0), 1))
		mn_wvs = np.mean(waveforms, 0)
		wvs = mn_wvs[best_chan, :]
		half_amp = np.max(wvs) / 2
		half_amp = np.zeros_like(wvs) + half_amp
		t = np.linspace(0, 1/1000., 50)
		# create functions from the data using PiecewisePolynomial
		p1 = interpolate.PiecewisePolynomial(t, wvs[:,np.newaxis])
		p2 = interpolate.PiecewisePolynomial(t, half_amp[:,np.newaxis])
		xs = np.r_[t, t]
		xs.sort()
		x_min = xs.min()
		x_max = xs.max()
		x_mid = xs[:-1] + np.diff(xs) / 2
		roots = set()
		for val in x_mid:
			root, infodict, ier, mesg = optimize.fsolve(lambda x: p1(x)-p2(x), val, full_output=True)
			if ier==1 and x_min < root < x_max:
				roots.add(root[0])
		roots = list(roots)
		if len(roots) > 1:
			r = np.abs(np.diff(roots[0:2]))[0]
		else:
			r = np.nan
		return r

	def p2t_time(self, cluster):
		"""
		The peak to trough time of a spike in ms
		
		Parameters
		----------
		cluster: int
			the cluster whose waveforms are to be analysed
			
		Returns
		-------
		p2t: float
			The mean peak-to-trough time for the channel (electrode) that has 
			the strongest (highest amplitude) signal. Units are ms
		"""
		waveforms = self.waveforms[self.cut==cluster, :, :]
		best_chan = np.argmax(np.max(np.mean(waveforms, 0), 1))
		tP = self.getParam(waveforms, param='tP')
		tT = self.getParam(waveforms, param='tT')
		mn_tP = np.mean(tP, 0)
		mn_tT = np.mean(tT, 0)
		p2t = np.abs(mn_tP[best_chan] - mn_tT[best_chan])
		return p2t * 1000

	def plotClusterSpace(self, clusters=None, param='Amp', clusts=None, bins=256, **kwargs):
		"""
		TODO: aspect of plot boxes in ImageGrid not right as scaled by range of
		values now
		"""
		import tintColours as tcols
		import matplotlib.colors as colors
		from itertools import combinations
		from mpl_toolkits.axes_grid1 import ImageGrid
		
		if isinstance(clusters, int):
			clusters = [clusters]

		amps = self.getParam(param=param)
		bad_electrodes = np.setdiff1d(np.array(range(4)),np.array(np.sum(amps,0).nonzero())[0])
		cmap = np.tile(tcols.colours[0],(bins,1))
		cmap[0] = (1,1,1)
		cmap = colors.ListedColormap(cmap)
		cmap._init()
		alpha_vals = np.ones(cmap.N+3)
		alpha_vals[0] = 0
		cmap._lut[:,-1] = alpha_vals
		cmb = combinations(range(4),2)
		if 'fig' in kwargs.keys():
			fig = kwargs['fig']
		else:
			fig = plt.figure(figsize=(8,6))
		if 'rect' in kwargs.keys():
			rect = kwargs['rect']
		else:
			rect = 111
		grid = ImageGrid(fig, rect, nrows_ncols= (2,3), axes_pad=0.1, aspect=False)
		if 'Amp' in param:
			myRange = np.vstack((self.scaling*0, self.scaling*2))
		else:
			myRange = None
		clustCMap0 = np.tile(tcols.colours[0],(bins,1))
		clustCMap0[0] = (1,1,1)
		clustCMap0 = colors.ListedColormap(clustCMap0)
		clustCMap0._init()
		clustCMap0._lut[:,-1] = alpha_vals
		for i, c in enumerate(cmb):
			if c not in bad_electrodes:
				h, ye, xe = np.histogram2d(amps[:,c[0]], amps[:,c[1]], range = myRange[:,c].T, bins=bins)
				x, y = np.meshgrid(xe[0:-1], ye[0:-1])
				grid[i].pcolormesh(x, y, h, cmap=clustCMap0, edgecolors='face')
				if clusters is not None:
					for thisclust in clusters:
						clustidx = self.cut == thisclust
						h, ye, xe = np.histogram2d(amps[clustidx,c[0]],amps[clustidx,c[1]], range=myRange[:,c].T, bins=bins)
						clustCMap = np.tile(tcols.colours[thisclust],(bins,1))
						clustCMap[0] = (1,1,1)
						clustCMap = colors.ListedColormap(clustCMap)
						clustCMap._init()
						clustCMap._lut[:,-1] = alpha_vals
						grid[i].pcolormesh(x, y, h, cmap=clustCMap, edgecolors='face')
			s = str(c[0]+1) + ' v ' + str(c[1]+1)
			grid[i].text(0.05,0.95, s, va='top', ha='left', size='small', color='k', transform=grid[i].transAxes)
			grid[i].set_xlim(xe.min(), xe.max())
			grid[i].set_ylim(ye.min(), ye.max())
		plt.setp([a.get_xticklabels() for a in grid], visible=False)
		plt.setp([a.get_yticklabels() for a in grid], visible=False)
		return fig

class SpikeCalcsProbe(SpikeCalcsGeneric):
	"""
	Encapsulates methods specific to probe-based recordings
	"""
	def __init__(self):
		pass
